﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using Shared;
using System.Threading;
using CMDLL = CMMSDK.SafeNativeMethods;


namespace Digital_IO
{
    public partial class Digital_IO : Form
    {
        int m_nNumInputCh = 8;
        int m_nNumOutputCh = 8;

        private static int MAX_TEST_CH_NUM = 8;

        Label[] lblLedInputChList = new Label[MAX_TEST_CH_NUM];
        Label[] lblInputChList = new Label[MAX_TEST_CH_NUM];
        Label[] lblLedOutputChList = new Label[MAX_TEST_CH_NUM];
        Label[] lblOutputChList = new Label[MAX_TEST_CH_NUM];
        
        public Digital_IO()
        {
            InitializeComponent();
        }

        private void Digital_IO_Load(object sender, EventArgs e)
        {
            // Load Motion & DIO devices //
            int nNumAxes = 0;
            if (CMDLL.cmmGnDeviceLoad((int)Defines._TCmBool.cmTRUE, ref nNumAxes) != Defines.cmERR_NONE)
            {
                CMDLL.cmmErrShowLast(this.Handle); // 에러 발생하였으면 에러 디스플레이합니다.
                return;
            }
            else
            {
                //////////////////////////////////////////////////////////////////////////
                // 화면상 UI 를 업데이트 합니다.
                //////////////////////////////////////////////////////////////////////////
                InitComponent();
                //////////////////////////////////////////////////////////////////////////
                
                // Set timer to display current command position //
                this.timer1.Start();
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            int nInStatus = 0;

            //////////////////////////////////////////////////////////////////////////
            // 전체 Input Channel 확인하여, Turn On 된 채널을 화면에 표시합니다.
            //////////////////////////////////////////////////////////////////////////
            CMDLL.cmmDiGetMulti(0, 8, ref nInStatus);
            for (int i = 0; i < 8; i++)
            {
                if (((nInStatus >> i) & 1) == 1)
                {
                    lblLedInputChList[i].BackColor = Color.Red;
                    lblLedInputChList[i].Text = "on";

                }
                else
                {
                    lblLedInputChList[i].BackColor = Color.Black;
                    lblLedInputChList[i].Text = "off";
                }
            }
        }

        void InitComponent()
        {
            //////////////////////////////////////////////////////////////////////////
            //	Input Channel 에 대한 채널 갯수를 화면에 업데이트 합니다.
            //////////////////////////////////////////////////////////////////////////
            // 가능한 입력 채널 수를 얻어 오기전에 모든 입력 채널 LED 및 Label 을 Disable 상태로 만듭니다.
            lblLedInputChList[0] = lblLedInputCh0;
            lblLedInputChList[1] = lblLedInputCh1;
            lblLedInputChList[2] = lblLedInputCh2;
            lblLedInputChList[3] = lblLedInputCh3;
            lblLedInputChList[4] = lblLedInputCh4;
            lblLedInputChList[5] = lblLedInputCh5;
            lblLedInputChList[6] = lblLedInputCh6;
            lblLedInputChList[7] = lblLedInputCh7;

            for (int i = 0; i < lblLedInputChList.Length; i++)
            {
                lblLedInputChList[i].Enabled = false;
            }

            lblInputChList[0] = lblInputCh0;
            lblInputChList[1] = lblInputCh1;
            lblInputChList[2] = lblInputCh2;
            lblInputChList[3] = lblInputCh3;
            lblInputChList[4] = lblInputCh4;
            lblInputChList[5] = lblInputCh5;
            lblInputChList[6] = lblInputCh6;
            lblInputChList[7] = lblInputCh7;

            for (int i = 0; i < lblInputChList.Length; i++)
            {
                lblInputChList[i].Enabled = false;
            }

            CMDLL.cmmAdvGetNumAvailDioChan((int)Defines._TCmBool.cmTRUE, ref m_nNumInputCh);
            this.txtAvailInput.Text = m_nNumInputCh.ToString();

            // 가능한 입력 채널의 LED 와 Label 을 Enable 상태로 만듭니다.
            if (m_nNumInputCh > 0)
            {
                this.lblLedInputCh0.Enabled = true;
                this.lblInputCh0.Enabled = true;

                this.lblLedInputCh1.Enabled = true;
                this.lblInputCh1.Enabled = true;

                this.lblLedInputCh2.Enabled = true;
                this.lblInputCh2.Enabled = true;
            }
            if (m_nNumInputCh > 3)
            {
                this.lblLedInputCh3.Enabled = true;
                this.lblInputCh3.Enabled = true;

                this.lblLedInputCh4.Enabled = true;
                this.lblInputCh4.Enabled = true;

                this.lblLedInputCh5.Enabled = true;
                this.lblInputCh5.Enabled = true;
            }
            if (m_nNumInputCh > 6)
            {
                this.lblLedInputCh6.Enabled = true;
                this.lblInputCh6.Enabled = true;

                this.lblLedInputCh7.Enabled = true;
                this.lblInputCh7.Enabled = true;
            }

            //////////////////////////////////////////////////////////////////////////
            //	Output Channel 에 대한 채널 갯수를 화면에 업데이트 합니다.
            //////////////////////////////////////////////////////////////////////////
            // 가능한 출력 채널 수를 얻어 오기전에 모든 출력 채널 버튼을 Disable 상태로 만듭니다.
            lblLedOutputChList[0] = lblLedOuputCh0;
            lblLedOutputChList[1] = lblLedOuputCh1;
            lblLedOutputChList[2] = lblLedOuputCh2;
            lblLedOutputChList[3] = lblLedOuputCh3;
            lblLedOutputChList[4] = lblLedOuputCh4;
            lblLedOutputChList[5] = lblLedOuputCh5;
            lblLedOutputChList[6] = lblLedOuputCh6;
            lblLedOutputChList[7] = lblLedOuputCh7;

            for (int i = 0; i < lblLedOutputChList.Length; i++)
            {
                lblLedOutputChList[i].Enabled = false;
            }

            lblOutputChList[0] = lblOuputCh0;
            lblOutputChList[1] = lblOuputCh1;
            lblOutputChList[2] = lblOuputCh2;
            lblOutputChList[3] = lblOuputCh3;
            lblOutputChList[4] = lblOuputCh4;
            lblOutputChList[5] = lblOuputCh5;
            lblOutputChList[6] = lblOuputCh6;
            lblOutputChList[7] = lblOuputCh7;

            for (int i = 0; i < lblOutputChList.Length; i++)
            {
                lblOutputChList[i].Enabled = false;
            }

            CMDLL.cmmAdvGetNumAvailDioChan((int)Defines._TCmBool.cmFALSE, ref m_nNumOutputCh);
            this.txtAvailOuput.Text = m_nNumOutputCh.ToString();

            // 가능한 출력 채널 버튼을 Enable 상태로 만듭니다.
            if(m_nNumOutputCh > 0)
            {
                this.lblLedOuputCh0.Enabled = true;
                this.lblLedOuputCh1.Enabled = true;
                this.lblLedOuputCh2.Enabled = true;

                this.lblOuputCh0.Enabled = true;
                this.lblOuputCh1.Enabled = true;
                this.lblOuputCh2.Enabled = true;
            }
            if(m_nNumOutputCh > 3)
            {
                this.lblLedOuputCh3.Enabled = true;
                this.lblLedOuputCh4.Enabled = true;
                this.lblLedOuputCh5.Enabled = true;

                this.lblOuputCh3.Enabled = true;
                this.lblOuputCh4.Enabled = true;
                this.lblOuputCh5.Enabled = true;
            }
            if (m_nNumOutputCh > 6)
            {
                this.lblLedOuputCh6.Enabled = true;
                this.lblLedOuputCh7.Enabled = true;

                this.lblOuputCh6.Enabled = true;
                this.lblOuputCh7.Enabled = true;
            }

            //////////////////////////////////////////////////////////////////////////
            // Output Channel 의 상태에 따라서 버튼의 상태 변경
            //////////////////////////////////////////////////////////////////////////
            SetOutputStatusBtn();
        }

        

        void SetOutputStatusBtn()
        {
            //////////////////////////////////////////////////////////////////////////
            // Output Channel 의 출력 상태를 초기에 현재 상태를 읽어와 화면에 표시합니다.
            //////////////////////////////////////////////////////////////////////////
            for (int i = 0; i < 8; i++)
            {
                int nOutputStatus = 0x0;
                int nOutputLogic = 0x0;

                if (CMDLL.cmmDoGetOne(i, ref nOutputStatus) == Defines.cmERR_NONE)
                {
                    CMDLL.cmmDoGetOutputLogic(i, ref nOutputLogic);
                    if (nOutputLogic == (int)Defines._TCmSigLogic.cmLOGIC_A)
                    {
                        SetPushButtonStatus(i, nOutputStatus);
                    }
                    else if (nOutputLogic == (int)Defines._TCmSigLogic.cmLOGIC_B)
                    {
                        //SetPushButtonStatus(i, nOutputStatus);
                    }
                }
            }
        }

        // 해당 Push 버튼의 상태를 변경합니다.
        void SetPushButtonStatus(int idx, int bState)
        {
            if (bState == 0)
            {
                lblLedOutputChList[idx].BackColor = Color.Black;
                lblLedOutputChList[idx].Text = "off";
            }
            else
            {
                lblLedOutputChList[idx].BackColor = Color.Red;
                lblLedOutputChList[idx].Text = "on";
            }
        }

        private void btnAllOn_Click(object sender, EventArgs e)
        {
            if (m_nNumOutputCh < 1)
            {
                return; // 디지탈 출력 기능을 갖는 보드가 1장도 장착되어 있지 않은 경우
            }

            if (m_nNumOutputCh <= 3) // LX502 1장만 장착된 경우
            {
                CMDLL.cmmDoPutMulti(0, 8, 0x07);
            }
            else if (m_nNumOutputCh <= 6) // LX502 2장만 또는 LX504 1장만 장착된 경우
            {
                CMDLL.cmmDoPutMulti(0, 8, 0xFF);
            }
            else // // 디지털 I/O 출력 채널이 8채널 이상 사용 가능한 경우 => 본 예제에서는 최대 8채널만 사용 합니다.
            {
                CMDLL.cmmDoPutMulti(0, 8, 0xFF);
            }

            // Output 상태에 따라서 버튼을 변경한다.
            SetOutputStatusBtn();
            
            // 참고: cmmDoPutMulti 함수 한번 호출 시 최대 32채널의 On/Off 상태를 동시에 설정할 수 있습니다.
            // 예: 0 ~ 34 채널 까지 디지털 출력 채널을 제어하는 경우 다음과 같이 함수를 두번 호출 하시면 됩니다.
            //                                         Ch31~Ch28 Ch27~Ch24 Ch23~Ch20 Ch19~Ch16 Ch15~Ch12 Ch11~Ch08 Ch07~Ch04 Ch03~Ch0
            //		cmmDoPutMulti(0, 32, 0xAAAA5555); [  1010       1010      1010      1010     0101       0101      0101     0101  ]
            //		cmmDoPutMulti(32, 3, 0x7); Ch32 ~ Ch34 까지 총 3채널을 모두 On [111(2진수) -> 0x7(16진수)]
        }

        private void btnAllOff_Click(object sender, EventArgs e)
        {
            CMDLL.cmmDoPutMulti(0, m_nNumOutputCh, 0x0);
            // Output 상태에 따라서 버튼을 변경한다.
            SetOutputStatusBtn();
        }

        private void lblLedOuputCh0_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if(CMDLL.cmmDoGetOne(0, ref nStatus) == Defines.cmERR_NONE)
            {
                if(nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(0, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[0].BackColor = Color.Black;
                    lblLedOutputChList[0].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(0, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[0].BackColor = Color.Red;
                    lblLedOutputChList[0].Text = "on";
                }
            }
        }

        private void lblLedOuputCh1_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(1, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(1, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[1].BackColor = Color.Black;
                    lblLedOutputChList[1].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(1, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[1].BackColor = Color.Red;
                    lblLedOutputChList[1].Text = "on";
                }
            }
        }

        private void lblLedOuputCh2_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(2, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(2, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[2].BackColor = Color.Black;
                    lblLedOutputChList[2].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(2, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[2].BackColor = Color.Red;
                    lblLedOutputChList[2].Text = "on";
                }
            }
        }

        private void lblLedOuputCh3_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(3, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(3, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[3].BackColor = Color.Black;
                    lblLedOutputChList[3].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(3, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[3].BackColor = Color.Red;
                    lblLedOutputChList[3].Text = "on";
                }
            }
        }

        private void lblLedOuputCh4_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(4, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(4, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[4].BackColor = Color.Black;
                    lblLedOutputChList[4].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(4, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[4].BackColor = Color.Red;
                    lblLedOutputChList[4].Text = "on";
                }
            }
        }

        private void lblLedOuputCh5_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(5, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(5, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[5].BackColor = Color.Black;
                    lblLedOutputChList[5].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(5, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[5].BackColor = Color.Red;
                    lblLedOutputChList[5].Text = "on";
                }
            }
        }

        private void lblLedOuputCh6_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(6, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(6, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[6].BackColor = Color.Black;
                    lblLedOutputChList[6].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(6, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[6].BackColor = Color.Red;
                    lblLedOutputChList[6].Text = "on";
                }
            }
        }

        private void lblLedOuputCh7_Click(object sender, EventArgs e)
        {
            int nStatus = 0;
            if (CMDLL.cmmDoGetOne(7, ref nStatus) == Defines.cmERR_NONE)
            {
                if (nStatus == 1)
                {
                    CMDLL.cmmDoPutOne(7, (int)Defines._TCmBool.cmFALSE);
                    lblLedOutputChList[7].BackColor = Color.Black;
                    lblLedOutputChList[7].Text = "off";
                }
                else
                {
                    CMDLL.cmmDoPutOne(7, (int)Defines._TCmBool.cmTRUE);
                    lblLedOutputChList[7].BackColor = Color.Red;
                    lblLedOutputChList[7].Text = "on";
                }
            }
        }

        private void Digital_IO_FormClosing(object sender, FormClosingEventArgs e)
        {
            // 사용중인 타이머 종료.
            timer1.Stop();

            CMDLL.cmmGnDeviceUnload(); // Unload Device
        }
    }
}
